/*
 *	S1C17702p^[OXL[ǂ
 *
 *	* Sun Feb 07 13:27:39 JST 2010 Naoyuki Sawa
 *	- 1st [XB
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "s1c17702.h"

/*****************************************************************************
 *
 *****************************************************************************/

unsigned long seed;
int errno;

int now;

#define UPDOWN_MOTION(i,n) (abs(((i)+((n)-2))%(((n)-1)*2)-((n)-2)))

float RNDF_RANGE(float a, float b) {
	float d = (float)rand() / (float)RAND_MAX;
	d *= (float)(b - a);
	d += (float)a;
	return d;
}

/*****************************************************************************
 *	PAD
 *****************************************************************************/

#define PAD_A		0x01		/* SW2 */
#define PAD_B		0x02		/* SW3 */
#define PAD_C		0x04		/* JP2 */
#define PAD_MASK	0x07
#define TRG_A		0x10		/* SW2 */
#define TRG_B		0x20		/* SW3 */
#define TRG_C		0x40		/* JP2 */
#define TRG_MASK	0x70

int pad;

/*---------------------------------------------------------------------------*/

void
pad_update()
{
	int val = ((P0_IN_P0IN >> 6) | ((P3_IN_P3IN & 1) << 2)) ^ 7;	/* 0 = on, 1 = off */
	int trg = (pad ^ val) & val;
	pad = val | (trg << 4);
}

/*****************************************************************************
 *	LED
 *****************************************************************************/

int led;	/* 0 = off, 1 = on */

/*---------------------------------------------------------------------------*/

void
led_update()
{
	if(led) {
		P2_OUT_P2OUT &= 0x7F;	/* 0 = on */
	} else {
		P2_OUT_P2OUT |= 0x80;	/* 1 = off */
	}
}

/*****************************************************************************
 *	LCD
 *****************************************************************************/

#define DISP_X		72		/* \̈ */
#define DISP_Y		32		/* \̈ */

unsigned char vbuff[DISP_Y][DISP_X];

extern unsigned char SRAM[(DISP_Y/*\̈*/ + 16/*gp̈*/) / 8][DISP_X/*\̈*/ + 24/*gp̈*/ + 160/*̈*/];
extern volatile int lcd_intc;

typedef struct _PBMP_FILEHEADER {
	unsigned long head;		// + 0,4
	unsigned long fsize;		// + 4,4
	unsigned char bpp;		// + 8,1
	unsigned char mask;		// + 9,1
	short w;			// +10,2
	short h;			// +12,2
	/* padding */			// +14,2
	unsigned long buf_size;		// +16,4
} PBMP_FILEHEADER;			// =20

enum {
	TEX_BG,
	TEX_PAT,
};

extern const unsigned char BG[];
extern const unsigned char PAT[];

const PBMP_FILEHEADER* const texture_table[] = {
	(PBMP_FILEHEADER*)BG,		// TEX_BG
	(PBMP_FILEHEADER*)PAT,		// TEX_PAT
};

/*---------------------------------------------------------------------------*/

void
lcd_update()
{
	int x;
	int y;
	int c;

	for(y = 0; y < DISP_Y; y += 8) {
		for(x = 0; x < DISP_X; x++) {
			c  = vbuff[y + 0][x];
			c |= vbuff[y + 1][x] << 1;
			c |= vbuff[y + 2][x] << 2;
			c |= vbuff[y + 3][x] << 3;
			c |= vbuff[y + 4][x] << 4;
			c |= vbuff[y + 5][x] << 5;
			c |= vbuff[y + 6][x] << 6;
			c |= vbuff[y + 7][x] << 7;
			SRAM[y / 8][x] = c;
		}
	}
}

void
lcd_draw(const PBMP_FILEHEADER* pbmp, int dx, int dy, int sx, int sy, int w, int h, int param)
{
	static const unsigned char mask[8] = { 0x80,0x40,0x20,0x10,0x08,0x04,0x02,0x01 };
	unsigned char* dbuf;
	unsigned char* sbuf;
	unsigned sadr, c1, c2;

	if(dx < 0) { w += dx; sx -= dx; dx = 0; }
	if(dx + w > DISP_X) { w = DISP_X - dx; }
	if(w <= 0) return;
	if(dy < 0) { h += dy; sy -= dy; dy = 0; }
	if(dy + h > DISP_Y) { h = DISP_Y - dy; }
	if(h <= 0) return;

	dbuf = &vbuff[dy][dx];
	sbuf = (unsigned char*)(pbmp + 1);
	sadr = pbmp->w * sy + sx;
	c1 =  param       & 3;
	c2 = (param >> 2) & 3;

	do {
		int x = w;
		do {
			int c = (sbuf[sadr >> 3] & mask[sadr & 7]) ? c1 : c2;
			if(c < 2) {
				*dbuf = c;
			}
			dbuf++;
			sadr++;
		} while(--x);
		dbuf = dbuf + DISP_X  - w;
		sadr = sadr + pbmp->w - w;
	} while(--h);
}

/*****************************************************************************
 *	SPRITE
 *****************************************************************************/

typedef struct _SPRITE {
	unsigned char texno;		// +0,1: eNX`ԍ
	unsigned char x, y;		// +1,2: W
	unsigned char w, h;		// +3,2: A
	         char cx, cy;		// +5,2: _ItZbg
	unsigned char param;		// +7,1: [1:0] = c1, [3:2] = c2
} SPRITE;				// =8

#define DRW_CLR(C1,C2) ((C1)|(C2)<<2)

#define SPRITE_(name,texno,x,y,w,h,cx,cy,param) name,
enum {
	#include "sprite.h"
};
#undef SPRITE_

#define SPRITE_(name,texno,x,y,w,h,cx,cy,param) {(texno),(x),(y),(w),(h),(cx),(cy),(param)},
const SPRITE sprite_table[] = {
	#include "sprite.h"
};
#undef SPRITE_

/*---------------------------------------------------------------------------*/

void
sprite_draw(int x, int y, int sprno, int param)
{
	const SPRITE* spr = &sprite_table[sprno];
	const PBMP_FILEHEADER* pbmp = texture_table[spr->texno];

	x -= spr->cx;
	y -= spr->cy;
	param ^= spr->param;

	lcd_draw(pbmp, x, y, spr->x, spr->y, spr->w, spr->h, param);
}

/*****************************************************************************
 *	
 *****************************************************************************/

void
init()
{
	MISC_FL_FLCYC = 4;	/* FLASHC read access cycle = 1 cycle */
	MISC_FL_SRCYC = 0;	/* SRAMC access cycle = 2 cycles (overclocking?) */

	PSC_CTL_PRUN = 1;	/* Prescaler run/stop control = Run */
	OSC_CTL_OSC3EN = 1;	/* OSC3 enable = Enable */
	OSC_SRC_HSCLKSEL = 1;	/* High-speed clock select = OSC3 */

	/* LED */
	P2_IO_P2IO |= 0x80;	/* P27 port output enable = Enable */

	/* LCD */
	OSC_LCLK_LCKDV = 3;	/* LCD clock division ratio select = HSCLK1/256 */
	OSC_LCLK_LCKSRC = 0;	/* LCD clock source select = HSCLK */
	OSC_LCLK_LCKEN = 1;	/* LCD clock enable = Enable */
	//
	LCD_CADJ_LC = 10;	/* LCD contrast adjustment = 10 */
	//
	LCD_DCTL_SEGREV = 0;	/* Segment output assignment control = Reverse */
	LCD_DCTL_COMREV = 0;	/* Common output assignment control = Reverse */
//	LCD_DCTL_DSPREV = 0;	/* Reverse display control = Reverse */
	LCD_DCTL_DSPC = 1;	/* LCD display control = Normal display */
	//
	ITC_LV3_ILV6 = 1;	/* LCD interrupt level = 1 */
	LCD_IMSK_FRMIE = 1;	/* Frame signal interrupt enable = Enable */
}

/*****************************************************************************
 *
 *****************************************************************************/

void
schedule()
{
	led_update();
	lcd_update();

	if(!(pad & PAD_C)) {	/* JP2V[g̓m[EFCg */
		static int cnt1;
		       int cnt2;
		for(;;) {
			cnt2 = lcd_intc;
			if(cnt2 - cnt1 >= 4/*15FPS*/) {
				break;
			}
			asm("halt");
		}
		cnt1 = cnt2;
	}

	pad_update();

	now++;
}

/*****************************************************************************
 *
 *****************************************************************************/

#define FPS		15			/* t[[g */
#define SEC(s)		((int)(FPS * (s)))	/* b->t[ϊ */

#define CELL_W		13			/* J */
#define CELL_X		 5			/* w,r̃J=0W */
#define HELI_Y		 3			/* wYW */
#define BILD_Y		11			/* rYW */

#define GEN_ITV_RATE	99			/* loC^[o[g */
#define WARN_TIME	SEC(3)			/* rx */
#define MISS_TIME	SEC(5)			/* r~X */

/*---------------------------------------------------------------------------*/

typedef struct _BILD {
	int stat;		/* 0 = l, 1 = l, 2 = l(r̂), 3 = (ЊQr̂) */
	int cnt;		/* ėpJE^ */
} BILD;

typedef struct _EFCT {
	int type;		/* 1 = , 2 = , 3 = j */
	int cnt;
	int x;			/* jЂ̏ꍇ͏8bit */
	int y;
	int vx;
	int vy;
} EFCT;
#define EFCT_MAX	16

typedef struct _GAME {
	int miss;		/* 1 = ~X */
	//
	int score;		/* XRA */
	int score_cnt;		/* XRA\ */
	//
	int gen_itv;		/* loC^[o * 100 (lo閈GEN_ITV_RATE[%]ɂ) */
	int gen_cnt;		/* lo҂JEg * 100 */
	//
	int heli_stat;		/* 0 = ʏ, 1 = ~, 2 = , 3 = NbV */
	int heli_cnt;		/* ėpJE^ */
	int heli_col;		/* wʒu -1(NbV), 0,1,2,3, 4(), 5(ENbV) */
	int heli_man;		/* ݂l */
	//
	BILD bild[5];		/* [0...3] = ЊQr, [4] = r */
	EFCT efct[EFCT_MAX];	/* GtFNg */
	//
	int snd_ptn;		/* TEhp^[ */
	int snd_cnt;		/* TEhis */
} GAME;
GAME game;

/*---------------------------------------------------------------------------*/

void demo_main();

void game_main();
void game_init();

void man_gen();

void score_add(int n);
void score_draw();

void heli_action();
void heli_draw();

void bild_action(int i_bild);
void bild_draw(int i_bild);

void efct_draw();

/*---------------------------------------------------------------------------*/

int
main()
{
	schedule(); /* Ñ{^gK̂Ă邽 */
	for(;;) {
		led = 0;
		demo_main();
		led = 1;
		game_main();
	}
}

/****************************************************************************
 *	demo_main
 ****************************************************************************/

void demo_main() {
	int i;
	int x;
	int y;

	i = game.score;
	memset(&game, 0, sizeof game);
	game.score = i;

	for(;;) {
		schedule();
		led = (now % 15) < 8;

		/* twi */
		sprite_draw(0, 0, SPR_BG, 0);

		/* r` */
		for(i = 0; i < 5; i++) {
			bild_draw(i);
		}

		/* ^Cg` */
		x = 19;
		y = 4;
		sprite_draw(x, y, SPR_TITLE_W, 0);
		sprite_draw(x, y, SPR_TITLE_B, 0);

		/* XRA` */
		score_draw();

		if(pad & (TRG_A | TRG_B)) {
			return; /* Q[Jn */
		}

		/* srand()̑ */
		seed++;
	}
}

/****************************************************************************
 *	game_main
 ****************************************************************************/

void game_main() {
	int t1 = 0;
	int t2 = 0;
	int i;

	game_init();

	for(;;) {
		schedule();
		led = !game.miss;

		/* ANV */
		for(i = 0; i < 5; i++) {
			bild_action(i);
		}
		heli_action();
		if(!game.miss) {
			man_gen();
		}

		/* ` */
		sprite_draw(0, 0, SPR_BG, 0);
		for(i = 0; i < 5; i++) {
			bild_draw(i);
		}
		heli_draw();
		efct_draw();
		if(game.miss || game.score_cnt) {
			game.score_cnt--;
			score_draw();
		}

		if(!game.miss) {
			if(t1++ < SEC(1.5)) {
				sprite_draw(19, 8, SPR_GETREADY_B, 0);
				sprite_draw(19, 8, SPR_GETREADY_W, 0);
			}
		} else {
			if(t2++ > SEC(1.5)) {
				sprite_draw(19, 8, SPR_GAMEOVER_B, 0);
				sprite_draw(19, 8, SPR_GAMEOVER_W, 0);
				if(pad & (TRG_A | TRG_B)) {
					return; /* ^Cg֖߂ */
				}
			}
		}
	}
}

/****************************************************************************
 *	game_init
 ****************************************************************************/

void game_init() {
	memset(&game, 0, sizeof game);

	game.score_cnt = SEC(3);

	game.gen_itv = SEC(5) * 100;
	game.gen_cnt = SEC(2) * 100;

	game.heli_col = 4;
}

/****************************************************************************
 *	lo
 ****************************************************************************/

/* lo */
void man_gen() {
	BILD* bild;
	int i;

	/* ܂^CAEgĂȂ΁Alo҂JEg炷 (100Pʂł邱Ƃɒ) */
	if(game.gen_cnt > 0) {
		game.gen_cnt -= 100;
	}
	/* lo҂JEgA^CAEgc */
	if(game.gen_cnt <= 0) {
		/* oIB܃w͏O */
		do {
			i = rand() % 4;
		} while(i == game.heli_col);
		bild = &game.bild[i];
		/* ̗̃rɐlȂ΁Ao */
		if(!bild->stat) {
			bild->stat = 1;
			bild->cnt = 0;
			{
				long tmp; /* 16bit CPU ΍ */
				/* C^[o炷 */
				tmp = game.gen_itv;
				tmp *= (long)GEN_ITV_RATE;
				tmp /= (long)100;
				game.gen_itv = tmp;
				/* ̑҂Ԃ߂ */
				tmp *= (long)((rand() % 50) + 50); /* (C^[o~50`100%ŗh炷Ƃɂ) */
				tmp /= (long)100;
				game.gen_cnt = tmp;
			}
		}
	}
}

/****************************************************************************
 *	XRA
 ****************************************************************************/

void score_add(int n) {
	game.score += n;
	if(game.score > 999) {
		game.score = 999;
	}
	game.score_cnt = SEC(3);
}

void score_draw() {
	int n = game.score;
	int x = 72 - 5;
	int y = 32 - 7;
	int i;

	do {
		i = n % 10;
		sprite_draw(x, y, SPR_NUM0_B + i, 0);
		sprite_draw(x, y, SPR_NUM0_W + i, 0);

		x -= 4;
		n /= 10;
	} while(n);
}

/****************************************************************************
 *	w
 ****************************************************************************/

/* wANV */
void heli_action() {
	BILD* bild;
	EFCT* efct;
	int i;
	int n;
	int dir;
	int spd;

	switch(game.heli_stat) {
	case 0: /* ʏ */
		if(game.miss) {
			return;
		}
		/* {^ꂽAֈړJnB
		 * ÖړĂȂ΁Äʒuł̋~΂B
		 */
		if(pad & TRG_B) {
			game.heli_col--;
			if(game.heli_col == -1) { /* NbV */
				game.miss = 1;
				game.heli_stat = 3;
				game.heli_cnt = 0;
				goto L_CRASH;
			}
			game.heli_cnt = 3;
		/* E{^ꂽAEֈړJnB
		 * ÖړĂȂ΁Äʒuł̋~΂B
		 */
		} else if(pad & TRG_A) {
			game.heli_col++;
			if(game.heli_col == 5) { /* ENbV */
				game.miss = 1;
				game.heli_stat = 3;
				game.heli_cnt = 0;
				goto L_CRASH;
			}
			game.heli_cnt = -3;
		/* EĂȂ΁c */
		} else {
			/* ړȂ΁Aړp */
			if(game.heli_cnt < 0) {
				game.heli_cnt++;
			} else if(game.heli_cnt > 0) {
				game.heli_cnt--;
			/* ړĂc */
			} else {
				bild = &game.bild[game.heli_col];
				/* ЊQȑȂ΁c */
				if((game.heli_col >= 0) && (game.heli_col <= 3)) {
					/* l2l݂邵ĂȂ΁c */
					if(game.heli_man < 2) {
						/* lc */
						if((bild->stat == 1) && (bild->cnt >= 5)) {
							game.heli_stat = 1; /* ~Jn */
							//game.heli_cnt = 0;  0ł
							goto L_ROPE;
						}
					}
				/* ȑȂ΁c */
				} else if(game.heli_col == 4) {
					/* l݂邵Ăc */
					if(game.heli_man) {
						game.heli_stat = 2; /* Jn */
						//game.heli_cnt = 0;  0
						goto L_ESCP;
					}
				}
			}
		}
		break;

	case 1: /* ~ */
		if(game.miss) {
			return;
		}
		game.heli_cnt++;
L_ROPE:
		bild = &game.bild[game.heli_col];
		switch(game.heli_col) {
		case 1: /* 1i */
			if(game.heli_man == 0) { /* 1l */
				if(game.heli_cnt == 4) {
					/* rhoĂ(oroĂ)~oƁAeNjJ{[iX */
					if(bild->cnt >= WARN_TIME) {
						score_add(1);
					}
					/* rl */
					bild->stat = 0;
				} else if(game.heli_cnt == 7) {
					/* ݂グ */
					game.heli_stat = 0;
					game.heli_cnt = 0;
					game.heli_man++;
					return; /* ܂ */
				}
			} else { /* 2l */
				if(game.heli_cnt == 0) {
					/* rhoĂ(oroĂ)~oƁAeNjJ{[iX */
					if(bild->cnt >= WARN_TIME) {
						score_add(1);
					}
					/* rl */
					bild->stat = 0;
				} else if(game.heli_cnt == 3) {
					/* ݂グ */
					game.heli_stat = 0;
					game.heli_cnt = 0;
					game.heli_man++;
					return; /* ܂ */
				}
			}
			break;
		case 2: /* 3i */
			if(game.heli_man == 0) { /* 1l */
				if(game.heli_cnt == 12) {
					/* rhoĂ(oroĂ)~oƁAeNjJ{[iX */
					if(bild->cnt >= WARN_TIME) {
						score_add(1);
					}
					/* rl */
					bild->stat = 0;
				} else if(game.heli_cnt == 23) {
					/* ݂グ */
					game.heli_stat = 0;
					game.heli_cnt = 0;
					game.heli_man++;
					return; /* ܂ */
				}
			} else { /* 2l */
				if(game.heli_cnt == 7) {
					/* rhoĂ(oroĂ)~oƁAeNjJ{[iX */
					if(bild->cnt >= WARN_TIME) {
						score_add(1);
					}
					/* rl */
					bild->stat = 0;
				} else if(game.heli_cnt == 18) {
					/* ݂グ */
					game.heli_stat = 0;
					game.heli_cnt = 0;
					game.heli_man++;
					return; /* ܂ */
				}
			}
			break;
		default/*0,3*/: /* 2i */
			if(game.heli_man == 0) { /* 1l */
				if(game.heli_cnt == 8) {
					/* rhoĂ(oroĂ)~oƁAeNjJ{[iX */
					if(bild->cnt >= WARN_TIME) {
						score_add(1);
					}
					/* rl */
					bild->stat = 0;
				} else if(game.heli_cnt == 15) {
					/* ݂グ */
					game.heli_stat = 0;
					game.heli_cnt = 0;
					game.heli_man++;
					return; /* ܂ */
				}
			} else { /* 2l */
				if(game.heli_cnt == 3) {
					/* rhoĂ(oroĂ)~oƁAeNjJ{[iX */
					if(bild->cnt >= WARN_TIME) {
						score_add(1);
					}
					/* rl */
					bild->stat = 0;
				} else if(game.heli_cnt == 10) {
					/* ݂グ */
					game.heli_stat = 0;
					game.heli_cnt = 0;
					game.heli_man++;
					return; /* ܂ */
				}
			}
			break;
		}
		break;

	case 2: /*  */
		if(game.miss) {
			return;
		}
		game.heli_cnt++;
L_ESCP:
		bild = &game.bild[game.heli_col];
		if(game.heli_cnt == 3) {
			/* rɐlu */
			bild->stat = game.heli_man;
			bild->cnt = 0;
			/* XRAZB
			 * lÂĂAlÂĂA1_Ƃ邱Ƃɂ܂B
			 * lÂAẅړZăXN܂ɁA_B
			 */
			score_add(1);
		} else if(game.heli_cnt == 4) {
			/*  */
			game.heli_stat = 0;
			game.heli_cnt = 0;
			game.heli_man = 0;
			return;
		}
		break;

	case 3: /* NbV */
		game.heli_cnt++;
L_CRASH:
		/* jА */
		if(game.heli_cnt == 1) {
			n = 0;
			for(i = 0; i < EFCT_MAX; i++) {
				efct = &game.efct[i];
				if(!efct->type) {
					static const char dir_tbl[64][2] = { /* t2bit,6bit */
						{0x40,0x00},{0x3F,0x06},{0x3E,0x0C},{0x3D,0x12},{0x3B,0x18},{0x38,0x1E},{0x35,0x23},{0x31,0x28},
						{0x2D,0x2D},{0x28,0x31},{0x23,0x35},{0x1E,0x38},{0x18,0x3B},{0x12,0x3D},{0x0C,0x3E},{0x06,0x3F},
						{0x00,0x40},{0xFA,0x3F},{0xF4,0x3E},{0xEE,0x3D},{0xE8,0x3B},{0xE2,0x38},{0xDD,0x35},{0xD8,0x31},
						{0xD3,0x2D},{0xCF,0x28},{0xCB,0x23},{0xC8,0x1E},{0xC5,0x18},{0xC3,0x12},{0xC2,0x0C},{0xC1,0x06},
						{0xC0,0x00},{0xC1,0xFA},{0xC2,0xF4},{0xC3,0xEE},{0xC5,0xE8},{0xC8,0xE2},{0xCB,0xDD},{0xCF,0xD8},
						{0xD3,0xD3},{0xD8,0xCF},{0xDD,0xCB},{0xE2,0xC8},{0xE8,0xC5},{0xEE,0xC3},{0xF4,0xC2},{0xFA,0xC1},
						{0x00,0xC0},{0x06,0xC1},{0x0C,0xC2},{0x12,0xC3},{0x18,0xC5},{0x1E,0xC8},{0x23,0xCB},{0x28,0xCF},
						{0x2D,0xD3},{0x31,0xD8},{0x35,0xDD},{0x38,0xE2},{0x3B,0xE8},{0x3D,0xEE},{0x3E,0xF4},{0x3F,0xFA},
					};
					efct->type = 3; /* j */
					dir = (rand() % 16) + 24;
					spd = (rand() % 3) + 1;
					efct->vx = (dir_tbl[dir][0] << 2) * spd;
					efct->vy = (dir_tbl[dir][1] << 2) * spd;
					if(game.heli_col == -1) {
						efct->x = 1 << 8;
						efct->y = 6 << 8;
						efct->vx = -efct->vx;
					} else {
						efct->x = 68 << 8;
						efct->y =  6 << 8;
					}
					if(++n == 12) {
						break;
					}
				}
			}
		}
		break;

//	default:
//		DIE();
	}
}

/* w` */
void heli_draw() {
	static const unsigned char ofs_tbl[4/*heli_cnt*/] = { 0,1,3,7 };

	int x;
	int spr;

	switch(game.heli_stat) {
	case 0: /* ʏ */
		/* wWvZ */
		x = CELL_X + (CELL_W * game.heli_col);
		if(game.heli_cnt < 0) {
			x -= ofs_tbl[-game.heli_cnt];
		} else if(game.heli_cnt > 0) {
			x += ofs_tbl[game.heli_cnt];
		}
		/* w{̕` */
		spr = SPR_HELIN_00 + (now & 3);
		sprite_draw(x, HELI_Y, spr, 0);
		/* ݂` */
		if(game.heli_man == 1) {
			spr = SPR_HANG1;
			sprite_draw(x, HELI_Y, spr, 0);
		} else if(game.heli_man == 2) {
			spr = SPR_HANG2;
			sprite_draw(x, HELI_Y, spr, 0);
		}
		break;

	case 1: /* ~ */
		/* wWvZ */
		x = CELL_X + (CELL_W * game.heli_col);
		/* w{̕` */
		spr = SPR_HELIN_00 + (now & 3);
		sprite_draw(x, HELI_Y, spr, 0);
		/* [v` */
		switch(game.heli_col) {
		case 1: /* 1i */
			if(game.heli_man == 0) { /* 1l */
				spr = SPR_ROPE11_00;
			} else { /* 2l */
				spr = SPR_ROPE12_00;
			}
			break;
		case 2: /* 3i */
			if(game.heli_man == 0) { /* 1l */
				spr = SPR_ROPE31_00;
			} else { /* 2l */
				spr = SPR_ROPE32_00;
			}
			break;
		default/*0,3*/: /* 2i */
			if(game.heli_man == 0) { /* 1l */
				spr = SPR_ROPE21_00;
			} else { /* 2l */
				spr = SPR_ROPE22_00;
			}
			break;
		}
		spr += game.heli_cnt;
		sprite_draw(x, HELI_Y, spr, 0);
		break;

	case 2: /*  */
		/* wWvZ */
		x = CELL_X + (CELL_W * game.heli_col);
		/* w{̕` */
		spr = SPR_HELIN_00 + (now & 3);
		sprite_draw(x, HELI_Y, spr, 0);
		/* [v` */
		if(game.heli_man == 1) { /* 1l */
			spr = SPR_RELS1_00;
		} else { /* 2l */
			spr = SPR_RELS2_00;
		}
		spr += game.heli_cnt;
		sprite_draw(x, HELI_Y, spr, 0);
		break;

	case 3: /* NbV */
		/* wWvZ */
		x = CELL_X + (CELL_W * game.heli_col);
		/* w{̕` */
		spr = (game.heli_col == -1) ? SPR_HELICL_00 : SPR_HELICR_00;
		spr += (game.heli_cnt <= 3) ? game.heli_cnt : 3;
		sprite_draw(x, HELI_Y, spr, 0);
		break;

//	default:
//		DIE();
	}
}

/****************************************************************************
 *	r
 ****************************************************************************/

/* rANV */
void bild_action(int i_bild) {
	BILD* bild = &game.bild[i_bild];
	EFCT* efct;
	int i;

	switch(bild->stat) {
	case 0: /* ʏ */
		/** no job **/
		break;

	case 1: /* l */
	case 2: /* l(r̂) */
		switch(i_bild) {
		case 0: /* ЊQr (΍, 2i) */
			/*  */
			if(bild->cnt >= WARN_TIME) {
				for(i = 0; i < EFCT_MAX; i++) {
					efct = &game.efct[i];
					if(!efct->type) {
						efct->type = 1; /*  */
						efct->cnt = 0;
						efct->x = CELL_X + CELL_W * i_bild;
						efct->y = 20;
						efct->x += rand() % 11;
						efct->y += rand() % 12;
						break;
					}
				}
			}
			/* FALLTHRU */
		case 1: /* ЊQr (nk, 3i) */
		case 2: /* ЊQr (nk, 1i) */
		case 3: /* ЊQr (nk, 2i) */
			bild->cnt++;
			if(bild->cnt >= MISS_TIME) {
				if(!game.miss) {
					game.miss = 1;
					bild->stat = 3; /*  */
				}
			}
			break;
		case 4: /* r */
			bild->cnt++;
			if(bild->cnt >= (4 * 2 * 2/*U*/) + (11 - 4)) {
				bild->stat = 0;
			}
			break;
//		default:
//			DIE();
		}
		break;

	case 3: /* (ЊQr̂) */
		switch(i_bild) {
		case 0: /* ЊQr (΍, 2i) */
			/*  */
			for(i = 0; i < EFCT_MAX; i++) {
				efct = &game.efct[i];
				if(!efct->type) {
					efct->type = 2; /*  */
					efct->cnt = 0;
					efct->x = CELL_X + CELL_W * i_bild;
					efct->y = 20;
					efct->x += rand() % 11;
					efct->y += rand() % 12;
					break;
				}
			}
			/* FALLTHRU */
		case 1: /* ЊQr (nk, 3i) */
		case 2: /* ЊQr (nk, 1i) */
		case 3: /* ЊQr (nk, 2i) */
			bild->cnt++;
			break;
//		default:
//			DIE();
		}
		break;

//	default:
//		DIE();
	}
}

/* r` */
void bild_draw(int i_bild) {
	BILD* bild = &game.bild[i_bild];
	int i;
	int x;
	int spr;

	x = CELL_X + CELL_W * i_bild;

	switch(bild->stat) {
	case 0: /* ʏ */
		switch(i_bild) {
		case 0: /* ЊQr (΍, 2i) */
			/** no job **/
			break;
		case 1: /* ЊQr (nk, 3i) */
			/* r */
			spr = SPR_BILD3_00;
			sprite_draw(x, BILD_Y, spr, 0);
			break;
		case 2: /* ЊQr (nk, 1i) */
			/* r */
			spr = SPR_BILD1_00;
			sprite_draw(x, BILD_Y, spr, 0);
			break;
		case 3: /* ЊQr (nk, 2i) */
			/* r */
			spr = SPR_BILD2_00;
			sprite_draw(x, BILD_Y, spr, 0);
			break;
		case 4: /* r */
			/** no job **/
			break;
//		default:
//			DIE();
		}
		break;

	case 1: /* l */
	case 2: /* l(r̂) */
		switch(i_bild) {
		case 0: /* ЊQr (΍, 2i) */
			/* l */
			spr = SPR_HELP2_00;
			if(bild->cnt < 5) {
				spr += bild->cnt;
			} else {
				spr += (((bild->cnt - 5) / 2) & 3) + 5;
			}
			sprite_draw(x, HELI_Y, spr, 0);
			break;
		case 1: /* ЊQr (nk, 3i) */
			if(bild->cnt >= WARN_TIME) {
				x += UPDOWN_MOTION(bild->cnt, 3) - 1;
			}
			/* r */
			spr = SPR_BILD3_00;
			sprite_draw(x, BILD_Y, spr, 0);
			/* l */
			spr = SPR_HELP3_00;
			if(bild->cnt < 5) {
				spr += bild->cnt;
			} else {
				spr += (((bild->cnt - 5) / 2) & 3) + 5;
			}
			sprite_draw(x, HELI_Y, spr, 0);
			break;
		case 2: /* ЊQr (nk, 1i) */
			if(bild->cnt >= WARN_TIME) {
				x += UPDOWN_MOTION(bild->cnt, 3) - 1;
			}
			/* r */
			spr = SPR_BILD1_00;
			sprite_draw(x, BILD_Y, spr, 0);
			/* l */
			spr = SPR_HELP1_00;
			if(bild->cnt < 5) {
				spr += bild->cnt;
			} else {
				spr += (((bild->cnt - 5) / 2) & 3) + 5;
			}
			sprite_draw(x, HELI_Y, spr, 0);
			break;
		case 3: /* ЊQr (nk, 2i) */
			if(bild->cnt >= WARN_TIME) {
				x += UPDOWN_MOTION(bild->cnt, 3) - 1;
			}
			/* r */
			spr = SPR_BILD2_00;
			sprite_draw(x, BILD_Y, spr, 0);
			/* l */
			spr = SPR_HELP2_00;
			if(bild->cnt < 5) {
				spr += bild->cnt;
			} else {
				spr += (((bild->cnt - 5) / 2) & 3) + 5;
			}
			sprite_draw(x, HELI_Y, spr, 0);
			break;
		case 4: /* r */
			spr = (bild->stat == 1) ? SPR_ESCP1_00 : SPR_ESCP2_00;
			if(bild->cnt < (4 * 2 * 2/*U*/)) {
				spr += (bild->cnt / 2) & 3;
			} else {
				spr += bild->cnt - (4 * 2 * 2/*U*/) + 4;
			}
			sprite_draw(x, HELI_Y, spr, 0);
			break;
//		default:
//			DIE();
		}
		break;

	case 3: /* (ЊQr̂) */
		switch(i_bild) {
		case 0: /* ЊQr (΍, 2i) */
			/* l */
			spr = SPR_HELP2_00;
			spr += ((bild->cnt / 2) & 3) + 5;
			sprite_draw(x, HELI_Y, spr, 0);
			break;
		case 1: /* ЊQr (nk, 3i) */
			if(bild->cnt >= WARN_TIME) {
				x += UPDOWN_MOTION(bild->cnt, 3) - 1;
			}
			i = bild->cnt - MISS_TIME;
			/* r */
			spr = SPR_BILD3_00 + (i <= 16 ? i : 16);
			sprite_draw(x, BILD_Y, spr, 0);
			/* l */
			spr = SPR_HELP3_00;
			spr += ((bild->cnt / 2) & 3) + 5;
			sprite_draw(x, HELI_Y + i, spr, 0);
			break;
		case 2: /* ЊQr (nk, 1i) */
			if(bild->cnt >= WARN_TIME) {
				x += UPDOWN_MOTION(bild->cnt, 3) - 1;
			}
			i = bild->cnt - MISS_TIME;
			/* r */
			spr = SPR_BILD1_00 + (i <= 8 ? i : 8);
			sprite_draw(x, BILD_Y, spr, 0);
			/* l */
			spr = SPR_HELP1_00;
			spr += ((bild->cnt / 2) & 3) + 5;
			sprite_draw(x, HELI_Y + i, spr, 0);
			break;
		case 3: /* ЊQr (nk, 2i) */
			if(bild->cnt >= WARN_TIME) {
				x += UPDOWN_MOTION(bild->cnt, 3) - 1;
			}
			i = bild->cnt - MISS_TIME;
			/* r */
			spr = SPR_BILD2_00 + (i <= 12 ? i : 12);
			sprite_draw(x, BILD_Y, spr, 0);
			/* l */
			spr = SPR_HELP2_00;
			spr += ((bild->cnt / 2) & 3) + 5;
			sprite_draw(x, HELI_Y + i, spr, 0);
			break;
//		default:
//			DIE();
		}
		break;

//	default:
//		DIE();
	}
}

/****************************************************************************
 *	GtFNg
 ****************************************************************************/

/* GtFNg` */
void efct_draw() {
	EFCT* efct;
	int i;
	int x;
	int y;
	int spr;

	for(i = 0; i < EFCT_MAX; i++) {
		efct = &game.efct[i];
		switch(efct->type) {
		case 1: /*  */
			x = efct->x;
			y = efct->y;
			spr = SPR_SMOK_00 + efct->cnt;
			sprite_draw(x, y, spr, 0);
			efct->cnt++;
			if(efct->cnt >= 4) {
				efct->type = 0;
			}
			break;
		case 2: /*  */
			x = efct->x;
			y = efct->y;
			spr = SPR_FIRE_00 + efct->cnt;
			sprite_draw(x, y, spr, 0);
			efct->cnt++;
			if(efct->cnt >= 6) {
				efct->type = 0;
			}
			break;
		case 3: /* j */
			x = efct->x >> 8;
			y = efct->y >> 8;
			if(((unsigned)x < DISP_X) && ((unsigned)y < DISP_Y)) {
				vbuff[y][x] = 1;
			}
			efct->x += efct->vx;
			efct->y += efct->vy;
			efct->vx = (efct->vx * 7) >> 3;
			efct->vy = (efct->vy * 7) >> 3;
			if((abs(efct->vx) < 64) && (abs(efct->vy) < 64)) {
				efct->type = 0;
			}
			break;
		}
	}
}

